/*
 * Copyright (c) 2011, Noel Lemouel
 * All rights reserved.
 */

#include <stdio.h>
#include "ui.h"

static unsigned int palette_32[] = {0xfffce88c, 0xffdcb45c, 0xff987c3c, 0xff4c3c1c};
static unsigned short palette_16[] = {0xfe71, 0xddab, 0x9be7, 0x49e3};

static unsigned char cursor_8x8[] = {0xe7, 0x81, 0x81, 0x00, 0x00, 0x81, 0x81, 0xe7};

/*
	11100111
	11000011
	00000000
	00000000
	11000011
	11100111
*/

/* no$gmb DMG colors */

// 0xfffce88c  /* 00 */
// 0xffdcb45c  /* 01 */
// 0xff987c3c  /* 10 */
// 0xff4c3c1c  /* 11 */

// 0xfe71  /* 00 */
// 0xddab  /* 01 */
// 0x9be7  /* 10 */
// 0x49e3  /* 11 */

int draw_line_16(unsigned short* fb, unsigned int x0, unsigned int y0, unsigned int x1, unsigned int y1)
{
	unsigned int i;

	if((x1-x0) > 0 && (y0 == y1))
	{
		for(i=0; i<(x1-x0); i++)
		{
			fb[(y0*SCX)+(x0+i)] = 0xffff;
		}
	}

	if((y1-y0) > 0 && (x0 == x1))
	{
		for(i=0; i<(y1-y0); i++)
		{
			fb[((y0+i)*SCX)+x0] = 0xffff;
		}
	}
}

int blit_tile_16(unsigned int t_num, unsigned short* fb, unsigned int x, unsigned int y)
{
	unsigned i, j, k;
	unsigned char t_data[16];

	read_VRAM_tile(t_num, t_data);

	j=0;
	for(i=0; i<8; i++)
	{
		for(k=0; k<8; k++)
		{
			fb[((i+y)*SCX)+(7-k)+x] = \
			palette_16[((t_data[j] >> k) & 0x01) | (((t_data[j+1] >> k) & 0x01) << 1) & 0xff];
		}

		j+=2;
	}

	return 0;
}

int blit_tile_2x_16(unsigned int t_num, unsigned short* fb, unsigned int x, unsigned int y)
{
	unsigned i, j, k;
	unsigned char t_data[16];

	read_VRAM_tile(t_num, t_data);

	j=0;
	for(i=0; i<16; i++)
	{
		for(k=8; k>0; k--)
		{
			fb[(i*SCX)+((k*2)-2)] = \
			fb[(i*SCX)+((k*2)-1)] = \
			palette_16[((t_data[j] >> (8-k)) & 0x01) | (((t_data[j+1] >> (8-k)) & 0x01) << 1) & 0xff];
		}

		if(i%2) j+=2;
	}

	return 0;
}

int blit_tile_8x_16(unsigned int t_num, unsigned short* fb, unsigned int x, unsigned int y)
{
	unsigned i, j, k, l, m;
	unsigned char t_data[16];

	read_VRAM_tile(t_num, t_data);

	j=0; l=0;
	for(i=0; i<64; i++)
	{
		for(k=8; k>0; k--)
		{
			for(m=0; m<8; m++)
			{
				fb[((y+i)*SCX)+((k*8)-(8-m)+x)] = \
				palette_16[((t_data[j] >> (8-k)) & 0x01) | (((t_data[j+1] >> (8-k)) & 0x01) << 1) & 0xff];
			}
		}

		l++; if(l==8) l=0, j+=2;
	}

	return 0;
}

int blit_tile_32(unsigned int t_num, unsigned int* fb, unsigned int x, unsigned int y)
{
	unsigned i, j;
	unsigned char t_data[16];

	read_VRAM_tile(t_num, t_data);

	j=0;
	for(i=0; i<8; i++)
	{
		fb[((i+x)*SCX)+7+y] = palette_32[((t_data[j] & 0x01) | ((t_data[j+1] & 0x01) << 1)) & 0xff];
		fb[((i+x)*SCX)+6+y] = palette_32[((t_data[j] >> 1) & 0x01) | (((t_data[j+1] >> 1) & 0x01) << 1) & 0xff];
		fb[((i+x)*SCX)+5+y] = palette_32[((t_data[j] >> 2) & 0x01) | (((t_data[j+1] >> 2) & 0x01) << 1) & 0xff];
		fb[((i+x)*SCX)+4+y] = palette_32[((t_data[j] >> 3) & 0x01) | (((t_data[j+1] >> 3) & 0x01) << 1) & 0xff];
		fb[((i+x)*SCX)+3+y] = palette_32[((t_data[j] >> 4) & 0x01) | (((t_data[j+1] >> 4) & 0x01) << 1) & 0xff];
		fb[((i+x)*SCX)+2+y] = palette_32[((t_data[j] >> 5) & 0x01) | (((t_data[j+1] >> 5) & 0x01) << 1) & 0xff];
		fb[((i+x)*SCX)+1+y] = palette_32[((t_data[j] >> 6) & 0x01) | (((t_data[j+1] >> 6) & 0x01) << 1) & 0xff];
		fb[((i+x)*SCX)+y] = palette_32[((t_data[j] >> 7) & 0x01) | (((t_data[j+1] >> 7) & 0x01) << 1) & 0xff];

		j+=2;
	}

	return 0;
}

int blit_tile_2x_32(unsigned int t_num, unsigned int* fb, unsigned int x, unsigned int y)
{
	unsigned i, j;
	unsigned char t_data[16];

	read_VRAM_tile(0, t_data);

	j=0;
	for(i=0; i<16; i++)
	{
		fb[(i*SCX)+14] = fb[(i*SCX)+15] = palette_32[((t_data[j] & 0x01) | ((t_data[j+1] & 0x01) << 1)) & 0xff];
		fb[(i*SCX)+12] = fb[(i*SCX)+13] = palette_32[((t_data[j] >> 1) & 0x01) | (((t_data[j+1] >> 1) & 0x01) << 1) & 0xff];
		fb[(i*SCX)+10] = fb[(i*SCX)+11] = palette_32[((t_data[j] >> 2) & 0x01) | (((t_data[j+1] >> 2) & 0x01) << 1) & 0xff];
		fb[(i*SCX)+8 ] = fb[(i*SCX)+9 ] = palette_32[((t_data[j] >> 3) & 0x01) | (((t_data[j+1] >> 3) & 0x01) << 1) & 0xff];
		fb[(i*SCX)+6 ] = fb[(i*SCX)+7 ] = palette_32[((t_data[j] >> 4) & 0x01) | (((t_data[j+1] >> 4) & 0x01) << 1) & 0xff];
		fb[(i*SCX)+4 ] = fb[(i*SCX)+5 ] = palette_32[((t_data[j] >> 5) & 0x01) | (((t_data[j+1] >> 5) & 0x01) << 1) & 0xff];
		fb[(i*SCX)+2 ] = fb[(i*SCX)+3 ] = palette_32[((t_data[j] >> 6) & 0x01) | (((t_data[j+1] >> 6) & 0x01) << 1) & 0xff];
		fb[(i*SCX)   ] = fb[(i*SCX)+1 ] = palette_32[((t_data[j] >> 7) & 0x01) | (((t_data[j+1] >> 7) & 0x01) << 1) & 0xff];

		if(i%2) j+=2;
	}

	return 0;
}

int draw_cursor_8x8_16(unsigned short* fb, unsigned int x, unsigned int y)
{
	unsigned i, j;

	for(i=0; i<8; i++)
	{
		for(j=0; j<8; j++)
		{
			fb[((i+y)*SCX)+j+x] = ((cursor_8x8[i] >> j) & 0x01) ? 0xf800 : fb[((i+y)*SCX)+j+x];
		}
	}

	return 0;
}

