/*
 * Copyright (c) 2011, Noel Lemouel
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#ifdef MACOSX
#include "SDL/include/SDL.h"
#else
#include "SDL.h"
#endif

#include "ui.h"

SDL_Surface *fb;

static unsigned char VRAM_tile_data[384*16];		/* $8000-97FF */
static unsigned int i_VRAM_tile_data = 0;

static unsigned char VRAM_bg_map_9800h[32*32];
static unsigned int i_VRAM_bg_map_9800h = 0;

static unsigned char VRAM_bg_map_9C00h[32*32];
static unsigned int i_VRAM_bg_map_9C00h = 0;

static int sc_open(FILE**, const char*, struct stat*);

void fb_update(void)
{
	SDL_UpdateRect(fb, 0, 0, SCX, SCY);
}

void add_VRAM_tile_data(unsigned char val)
{
	if(i_VRAM_tile_data >= sizeof(VRAM_tile_data))
	{
		printf("VRAM_tile_data overflow [%d]\n", i_VRAM_tile_data);
	}

	VRAM_tile_data[i_VRAM_tile_data] = val;
	i_VRAM_tile_data ++;
}

void add_VRAM_bg_map_9800h(unsigned char val)
{
	if(i_VRAM_bg_map_9800h >= sizeof(VRAM_bg_map_9800h))
	{
		printf("VRAM_bg_map_9800h overflow [%d]\n", i_VRAM_bg_map_9800h);
	}

	VRAM_bg_map_9800h[i_VRAM_bg_map_9800h] = val;
	i_VRAM_bg_map_9800h ++;
}

void read_VRAM_tile(unsigned int t_num, unsigned char* t_data)
{
	unsigned int i;

	// printf("%d\n", t_num);

	for(i=0; i<16; i++)
	{
		// printf("%02x|", t_data[i]);
		t_data[i] = VRAM_tile_data[(t_num * 16)+i];
	}
	// printf("\n");
}

unsigned char read_VRAM_bg_map_9800h(unsigned int i)
{
	return VRAM_bg_map_9800h[i];
}

int main(int argc, char *argv[])
{

	unsigned int i;

	unsigned char* pxl_24;

	unsigned int pos_x;
	unsigned int pos_y;

	FILE* fd0;
	struct stat info;

	unsigned char* fd0_data;

	/* check # of arguments */

	if(argc < 2){
		printf("usage: %s VRAM_tile_data.inc [VRAM_bg_map_9800h.inc]\n", basename(argv[0]));
		return -1;
	}

	/* clear VRAM sections */

	for(i=0; i<sizeof(VRAM_tile_data); i++) VRAM_tile_data[i] = 0x00;
	for(i=0; i<sizeof(VRAM_bg_map_9800h); i++) VRAM_bg_map_9800h[i] = 0x00;
	for(i=0; i<sizeof(VRAM_bg_map_9C00h); i++) VRAM_bg_map_9C00h[i] = 0x00;

	/* open and load VRAM_tile_data file (argv[1]) */

	if( sc_open(&fd0, argv[1], &info) ) return -1;

	fd0_data = (unsigned char*)malloc(info.st_size);
	if(!fd0_data){
		printf("error: [alloc. %d] failed\n", (int)info.st_size);
		return -1;
	}else{
		printf("[alloc. %d]\n", (int)info.st_size);
	}

	fread(fd0_data, 1, info.st_size, fd0);		/* load fd0_data */

	parse(fd0_data, (int)info.st_size, add_VRAM_tile_data);

	free(fd0_data);
	fclose(fd0);

	/* open and load VRAM_bg_map_9800h file if present (argv[2]) */

	if(argc > 2)
	{
		if( sc_open(&fd0, argv[2], &info) ) return -1;

		fd0_data = (unsigned char*)malloc(info.st_size);
		if(!fd0_data){
			printf("error: [alloc. %d] failed\n", (int)info.st_size);
			return -1;
		}else{
			printf("[alloc. %d]\n", (int)info.st_size);
		}

		fread(fd0_data, 1, info.st_size, fd0);		/* load fd0_data */

		parse(fd0_data, (int)info.st_size, add_VRAM_bg_map_9800h);

		free(fd0_data);
		fclose(fd0);
	}

	/* look for additional args => not used yet */

	if(argc > 3)
	{
		printf("ignoring remaining args\n");
	}

	/* ... */

	/* Initialize SDL's subsystems - in this case, only video. */
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}

	/* Register SDL_Quit to be called at exit; makes sure things are
		cleaned up when we quit. */
	atexit(SDL_Quit);

#ifdef UI_RGB32
	/* Attempt to create a SCXxSCY window with 32bit per pixels. */
	fb = SDL_SetVideoMode(SCX, SCY, 32, SDL_SWSURFACE);
#endif

#ifdef UI_RGB16
	/* Attempt to create a SCXxSCY window with 16bit per pixels. */
	fb = SDL_SetVideoMode(SCX, SCY, 16, SDL_SWSURFACE);
#endif

	/* If we fail, return error. */
	if ( fb == NULL )
	{
		fprintf(stderr, "Unable to set %dx%d video: %s\n", SCX, SCY, SDL_GetError());
		exit(1);
	}

	SDL_WM_SetCaption ("objview", NULL);

	/* ... */

//	v_tile_data_init_16((Uint16*)fb->pixels);
//	v_bg_map_9800h_init_16((Uint16*)fb->pixels);

	/* ... */

	ui(fb);

	return 0;
}

static int sc_open(FILE** fd, const char* name, struct stat* info)
{
	if (stat(name, info) == -1)
	{
		if (errno == ENOENT)
		{
			(void) fprintf( stderr, "error: '%s' does not exist\r\n", name );
		}else{
			(void) fprintf( stderr, "error: stat() fails for '%s'\r\n", name );
		}
		return -1;
	}

	/* checking S_ISLNK(m) is not required since 'stat' follows sym. links */
	if (!S_ISREG(info->st_mode))
	{
		(void) fprintf( stderr, "error: '%s' is a not a regular file (or a sym. link)\r\n", name);
		return -1;
	}

	if ((*fd = fopen(name, "rb")) == NULL)
	{
		(void) fprintf( stderr, "error: '%s' open failed\r\n", name );
		return -1;
	}

	/* printf("size: %d\n", (int)info->st_size); */

	return 0;   /* no error */
}

