/*
 * Copyright (c) 2011, Noel Lemouel
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* generic array add function pointer */
void (*padd)(unsigned char) = NULL;

/* expects 0xdeadbeef or deadbeef */
unsigned int atox(unsigned char *ip_str)
{
	unsigned int op_val=0, i=0, ip_len = strlen((char*)ip_str);

	if(strncmp((char*)ip_str, "0x", 2) == 0){
		ip_str +=2;
		ip_len -=2;
	}

	for(i=0;i<ip_len;i++){
		op_val *= 0x10;
		switch(ip_str[i]){
			case 'A':
			case 'a':
				op_val += 0xa;
				break;
			case 'B':
			case 'b':
				op_val += 0xb;
				break;
			case 'C':
			case 'c':
				op_val += 0xc;
				break;
			case 'D':
			case 'd':
				op_val += 0xd;
				break;
			case 'E':
			case 'e':
				op_val += 0xe;
				break;
			case 'F':
			case 'f':
				op_val += 0xf;
				break;
			case '0' ... '9':
				op_val += (ip_str[i] - 0x30);
				break;
			default :
				op_val += 0x0;
				break;
		}
	}

	return op_val;
}

int parse(unsigned char* txt, unsigned int size, void (*padd)(unsigned char))
{
	unsigned int i, j, k, l;
	unsigned int line;
	unsigned char** pline;

	/* count the number of lines */

	line = 0;

	for(i=0; i<size; i++)
	{
		if(txt[i] == '\n') line++;
	}

	// printf("%d line(s)\n", line);

	/* create a table of pointers to lines */

	pline = malloc((line+1) * sizeof(unsigned char*));
	if(!pline)
	{
		printf("parse error [pline alloc.]\n");
		return 0;
	}

	j = 1;
	pline[0] = &txt[0];

	/* last pline[] entry is invalid */
	for(i=0; i<size; i++)
	{
		if(txt[i] == '\r' || txt[i] == ';') txt[i] = '\0';
		if(txt[i] == '\n') /* 0x0a */
		{
			pline[j++] = &txt[i+1];
			txt[i] = '\0';
		}
	}

	/* parse each line to extract hex data */

	for(i=0; i<line; i++)
	{
		j = 0;
		k = 0;

		/* look for ".db " */

		while(1)
		{
			if(pline[i][j] == '\0') break;		/* empty line */
			else if(pline[i][j] == ' ');			/* space => ignore */
			// else if(pline[i][j] == ';') break;	/* comment => ignore the rest of the line */
			else if(pline[i][j] == '.'){
				if(!strncmp((char*)&pline[i][j], ".db ", 4))	/* '.db ' ? */
				{
					j+=4; k=1;
					break;
				}else{
					printf("parse error at line %d\n", i+1);
					return -1;
				}
			}else{
				printf("parse error at line %d\n", i+1);
				return -1;
			}
			j++;
		}

		/* extract hex values */

		unsigned char hex[3];

		if(k)
		{

			// printf("'.db ' at line %d\n", i+1);

			l = 0;
			while(1)
			{
				if(pline[i][j] == ' ');			/* space => ignore */
				else if(((pline[i][j] == '$') && (l==0)) || ((pline[i][j] == ',') && (l>0)))
				{
					if(l==0)
					{
						j++;
						while(1)
						{
							if( (pline[i][j] >= 0x30 && pline[i][j] <= 0x39) ||			/* 0-9 */
								 (pline[i][j] >= 0x41 && pline[i][j] <= 0x46) ||			/* A-F */
								 (pline[i][j] >= 0x61 && pline[i][j] <= 0x66) ) break;	/* a-f */
							else{
								printf("parse error at line %d:%d [hex value expected]\n", i+1, j+1);
								return -1;
							}
							j++;
						}
 					}else{
						j++;
						while(1)
						{
							if(pline[i][j] == ' ');
							else if(pline[i][j] == '$')
							{
								j++;
								if( (pline[i][j] >= 0x30 && pline[i][j] <= 0x39) ||			/* 0-9 */
									 (pline[i][j] >= 0x41 && pline[i][j] <= 0x46) ||			/* A-F */
									 (pline[i][j] >= 0x61 && pline[i][j] <= 0x66) ) break;	/* a-f */
								else{
									printf("parse error at line %d:%d [hex value expected]\n", i+1, j+1);
									return -1;
								}
							}else{
								printf("parse error at line %d:%d\n", i+1, j+1);
								return -1;
							}
							j++;
						}
					}
					k=0;
					while( (pline[i][j] >= 0x30 && pline[i][j] <= 0x39) ||			/* 0-9 */
							 (pline[i][j] >= 0x41 && pline[i][j] <= 0x46) ||			/* A-F */
							 (pline[i][j] >= 0x61 && pline[i][j] <= 0x66) )				/* a-f */
					{
						if(k<2) hex[k] = pline[i][j];
						k++; j++;
					}
					l = 1;	/* at least 1 record after ".db" */
					if(k > 2) printf("hex value truncated to byte len at line %d:%d\n", i+1, j-k);
					hex[k] = '\0'; j--;
					// printf("%02x|", atox(hex));
					padd(atox(hex));
					/* ... */
				}
				else if(pline[i][j] == '\0') break;		/* end of line */
				else{
					printf("parse error at line %d:%d\n", i+1, j+1);
					return -1;
				}
				j++;
			}
			if(!l){
				printf("parse error at line %d [expected at least 1 record after \".db\" statement]\n", i+1);
			}
			// printf("\n");
		}

	}

	free(pline);

}

